@@ -31,7 +31,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL license and that you accept its terms.
 # -----------------------------------------------------------------------------
-'''
+"""
 DifferentialEquation class.
 
 The DifferentialEquation class allows to manipulate equations of the type:
@@ -49,19 +49,19 @@ the case, a `DifferentialEquationError` is raised.
 >>> eq = DifferentialEquation('dy/dt = a+b*y')
 >>> y = eq.evaluate(y, 0.01, 1, 2)     # a=1, b=2
 >>> y = eq.evaluate(y, 0.01, b=1, a=2) # a=2, b=1
-'''
-
+"""
 import re
-import numpy as np
 import inspect
-from definition import Definition
+import compiler
+import numpy as np
+from definition import Definition, Visitor
 
 
 class DifferentialEquationError(Exception):
     pass
 
 class DifferentialEquation(Definition):
-    ''' DifferentialEquation of type: 'dy/dt = expr' 
+    """ DifferentialEquation of type: 'dy/dt = expr'
 
     The DifferentialEquation class allows to manipulate equations of the type:
 
@@ -78,220 +78,241 @@ class DifferentialEquation(Definition):
     >>> eq = DifferentialEquation('dy/dt = a+b*x')
     >>> y = eq.evaluate(0.0, dt=0.1, a=1, b=2)
     0.1
-    '''
+    """
 
-    def __init__(self, definition):
-        '''
-        Creates equation if `definition` is of the right form.
+    def __init__(self, definition, constants=None):
+        """
+        Creates differential equation.
 
-        **Parameters**
+        :param string definition:
+            Equation definition
 
-        definition : str
-            Equation definition of the form 'dy/dt = expr : dtype'
-            expr must be a valid python expression.
-        '''
+        :param list constants:
+            Name of variables that must be considered constant
+        """
+        if not constants: constants = {}
         Definition.__init__(self, definition)
-        self.parse()
+        self._in_out = None
+        self._out = None
+        self.setup()
         self.__method__ = self._forward_euler
 
-
     def __repr__(self):
-        ''' x.__repr__() <==> repr(x) '''
+        """ x.__repr__() <==> repr(x) """
 
         classname = self.__class__.__name__
         return "%s('d%s/dt = %s : %s')" % (classname, self._lhs, self._rhs, self._dtype)
 
-
-    def parse(self, definition = None, known_variables = []):
-        '''
+    def setup(self, constants=None):
+        """
         Parse definition and check if it is an equation.
 
-        **Parameters**
+        :param string definition:
+            Equation definition of the form:
 
-        definition : str
-            Equation definition of the form 'y = expr : dtype'
-            expr must be a valid python expression.
-        '''
-        if definition is not None:
-            self._definition = definition
-        definition = str(self._definition.replace(' ',''))
-        # First, we check if equation is of the form:
-        #   dy/dt = A + (B)*y [: dtype]
-        p = re.compile(r'''d(?P<y>\w+)/dt =
-                           (?P<A>.+?)? (?P<sign>\+|-)? \((?P<B>.*)\)\*(?P=y)
+             dy/dt = expr : dtype or dy/dt = A+(B)*y : dtype
+
+            expr, A and B must be valid python expressions.
+
+        :param list constants:
+            List of variable names that must be considered constants.
+        """
+        if not constants: constants = {}
+
+        # First, we check if equation is of the form: dy/dt = A + (B)*y [: dtype]
+        # -----------------------------------------------------------------------
+        p = re.compile(
+            r'''d(?P<y>\w+)/dt = (?P<A>.+?)? (?P<sign>\+|-)? \((?P<B>.*)\)\*(?P=y)
                            (: (?P<dtype>.+))?''',re.VERBOSE)
-        result = p.match(definition)
+        result = p.match(self._definition)
         if result:
-            y = result.group('y')
-            A  = result.group('A') or '0'
-            sign = result.group('sign') or '+'
-            B = result.group('B')
+            y = result.group('y').strip()
+            A = (result.group('A') or '0').strip()
+            sign = (result.group('sign') or '+').strip()
+            B = result.group('B').strip()
             if A == '-' or A == '+':
-                sign = A
-                A = '0'
+                sign, A = A, '0'
+            if sign == '+':
+                B = '-' + B
             dtype = (result.group('dtype') or 'float')
 
-            # Check that var is not in A nor B
+
+            # Check that var is not in A nor in B
             if (y not in compile(A,'<string>', 'eval').co_names
                 and y not in compile(B,'<string>', 'eval').co_names):
+                self._lhs = y
                 self._varname = y
-                self._lhs = 'd%s/dt' % y
-                self.__f__ = eval('lambda %s: %s+%s*%s' % (y, A, B, y))
+                self._rhs = '%s%s(%s)*%s' % (A,sign,B,y)
+                self._dtype = dtype
+                visitor = Visitor()
+                compiler.walk(compiler.parse('%s+%s' % (A,B)), visitor)
+                variables = visitor._vars
 
-                # This line get all strings from function code definition...
-                self._variables = list(self.__f__.func_code.co_names)
-                #  ... and we need to sort out actual variables from function names
-                numpy_ns = {}
-                for key in dir(np):
-                    numpy_ns[key] = getattr(np,key)
-                for i in range(0,len(inspect.stack())):
-                    frame = inspect.stack()[i][0]
-                    for name in self.__f__.func_code.co_names:
-                        if ((name in self._variables) and (name != self._varname) and
-                            (name not in known_variables) and
-                            (name in numpy_ns.keys() or name in frame.f_globals.keys()) and
-                            callable(eval(name, numpy_ns, frame.f_globals))):
-                            self._variables.remove(name)
-                            numpy_ns[name] = frame.f_globals[name]
-
-                args = self.__f__.func_code.co_names
-                if len(args):
-                    args = ' = 0, '.join(self._variables)+ ' = 0'
+                ns = {}
+                for name in visitor._funcs:
+                    for i in range(1,len(inspect.stack())):
+                        frame = inspect.stack()[i][0]
+                        name = name.split('.')[0]
+                        if name in frame.f_locals.keys() and name not in ns.keys():
+                            ns[name] = frame.f_locals[name]
+                            break
+                ns.update(constants)
+                if y in variables:
+                    variables.remove(y)
+                variables = list(set(variables) - set(constants.keys()))
+                self._variables = variables
+                if len(variables):
+                    args = ' = 0, '.join(variables)+ ' = 0'
                 else:
                     args = ''
-                self.__f__ = eval('lambda %s,%s: %s+%s*%s' % (y,args,A,B,y), numpy_ns)
-                self._rhs = '%s+%s*%s' % (A,B,y)
-                self.__A__ = eval('lambda : %s+%s' % (A,B), numpy_ns)
-                args = self.__A__.func_code.co_names
-                if len(args):
-                    args = ' = 0, '.join(args)+ ' = 0'
-                else:
-                    args = ''
-                self.__A__ = eval('lambda %s: %s' % (args,A), numpy_ns)
+                self.__f__ = eval('lambda %s,%s: %s-%s*%s' % (y,args,A,B,y),ns)
+                self.__A__ = eval('lambda %s: %s' % (args,A),ns)
+                self.__B__ = eval('lambda %s: %s' % (args,B),ns)
                 self._A_string = A
-                self.__B__ = eval('lambda %s: %s' % (args,B), numpy_ns)
                 self._B_string = B
                 self._dtype = dtype
-                #self._variables = self._f.func_code.co_names
                 return
 
         # Second, we check if equation is of the form: dy/dt = f(y)
-        p = re.compile(r'''d(?P<y>\w+)/dt =
-                           (?P<f>[^:]+)
-                           (: (?P<dtype>\w+))?''', re.VERBOSE)
-        result = p.match(definition)
+        # ---------------------------------------------------------
+        p = re.compile(
+            r"d(?P<y>\w+)/dt = (?P<f>[^:]+) (: (?P<dtype>\w+))?", re.VERBOSE)
+        result = p.match(self._definition)
         if result:
             y = result.group('y').strip()
             f = result.group('f').strip()
             dtype = (result.group('dtype') or 'float').strip()
-            self._varname = y
             self._lhs = y
             self._rhs = f
-            self.__f__ = eval('lambda %s: %s' % (y,f))
-            # This line get all strings from function code definition...
-            self._variables = list(self.__f__.func_code.co_names)
-            #  ... and we need to sort out actual variables from function names
-            numpy_ns = {}
-            for key in dir(np):
-                numpy_ns[key] = getattr(np,key)
-            for i in range(0,len(inspect.stack())):
-                frame = inspect.stack()[i][0]
-                for name in self.__f__.func_code.co_names:
-                    if ((name in self._variables) and (name != self._varname) and
-                        (name not in known_variables) and
-                        (name in numpy_ns.keys() or name in frame.f_globals.keys()) and
-                        callable(eval(name, numpy_ns, frame.f_globals))):
-                        self._variables.remove(name)
-                        numpy_ns[name] = frame.f_globals[name]
-            args = self.__f__.func_code.co_names
-            if len(args):
-                args = ' = 0, '.join(self._variables)+ ' = 0'
+            self._dtype = dtype
+            self._varname = y
+            visitor = Visitor()
+            compiler.walk(compiler.parse(f), visitor)
+            variables = visitor._vars
+
+            ns = {}
+            for name in visitor._funcs:
+                for i in range(1,len(inspect.stack())):
+                    frame = inspect.stack()[i][0]
+                    name = name.split('.')[0]
+                    if name in frame.f_locals.keys() and name not in ns.keys():
+                        ns[name] = frame.f_locals[name]
+                        break
+            ns.update(constants)
+            if y in variables:
+                variables.remove(y)
+            variables = list(set(variables) - set(constants.keys()))
+            self._variables = variables
+            if len(variables):
+                args = ' = 0, '.join(variables)+ ' = 0'
             else:
                 args = ''
-            self.__f__ = eval('lambda %s,%s: %s' % (y,args,f), numpy_ns)
+            self.__f__ = eval('lambda %s,%s: %s' % (y,args,f), ns)
+            self._f_string = 'lambda %s,%s: %s' % (y,args,f)
             self.__A__ = None
-            self._A_string = ''
             self.__B__ = None
-            self._B_string = ''
-            self._dtype = dtype
+            self._A_string = ""
+            self._B_string = ""
             return
 
         # Last case, it is not a differential equation
+        # --------------------------------------------
         raise DifferentialEquationError, \
             'Equation is not a first order differential equation'
 
-
     def _forward_euler(self, __x__, dt, *args, **kwargs):
-        '''
+        """
         Forward euler method evaluation method
 
         **Notes:**
 
         See ``evaluate`` method for parameters
-        '''
-        __x__ += self.__f__(__x__, *args, **kwargs)*dt
-        return __x__
-
+        """
+        dx = self.__f__(__x__, *args, **kwargs)*dt
+        if self._out is not None:
+            np.add(__x__, dx, self._out)
+            return self._out
+        elif self._in_out is not None:
+            __x__ += dx
+            return __x__
+        return __x__ + dx
 
     def _runge_kutta_2(self, __x__, dt, *args, **kwargs):
-        '''
+        """
         Runge Kutta 2nd order evaluation method.
 
         **Notes**
 
         See ``evaluate`` method for parameters.
-        '''
+        """
         __hdt = 0.5*dt
         __k1 = self.__f__(__x__, *args, **kwargs)
         __k2 = self.__f__(__x__ + dt*__x__, *args, **kwargs)
-        __x__ += 0.5*dt*(__k1 + __k2)
-        return __x__
-
+        dx = 0.5*dt*(__k1 + __k2)
+        if self._out is not None:
+            np.add(__x__, dx, self._out)
+            return self._out
+        elif self._in_out is not None:
+            __x__ += dx
+            return __x__
+        return __x__ + dx
 
     def _runge_kutta_4(self, __x__, dt, *args, **kwargs):
-        '''
+        """
         Runge Kutta 4th order evaluation method.
 
         **Notes**
 
         See ``evaluate`` method for parameters.
-        '''
+        """
         __hdt = 0.5*dt
         __k1 = self.__f__(__x__, *args, **kwargs)
         __k2 = self.__f__(__x__ + __k1 * __hdt, *args, **kwargs)
         __k3 = self.__f__(__x__ + __k2 * __hdt, *args, **kwargs)
         __k4 = self.__f__(__x__ + __k3 * dt, *args, **kwargs)
-        __x__ += (__k1+__k4)*(dt/6.0)+(__k2+__k3)*(dt/3.0)
-        return __x__
-
+        dx  = (__k1+__k4)*(dt/6.0)+(__k2+__k3)*(dt/3.0)
+        if self._out is not None:
+            np.add(__x__, dx, self._out)
+            return self._out
+        elif self._in_out is not None:
+            __x__ += dx
+            return __x__
+        return __x__ + dx
 
     def _exponential_euler(self, __x__, dt, *args, **kwargs):
-        '''
+        """
         Exponential Euler evaluation method.
 
         **Notes**
 
         See ``evaluate`` method for parameters.
         Only available for equation of the form dy/dt = A + B*y
-        '''
-        A = self.__A__(*args, **kwargs)
-        B = self.__B__(*args, **kwargs)
-        AB = A/B
-        E = np.exp(B*dt)
-        __x__ *=  E
-        __x__ -=  AB
-        AB *= E          
-        __x__ += AB
+        """
+        A = float(self.__A__(*args, **kwargs))
+        B = float(self.__B__(*args, **kwargs))
+        AB = A
+        AB /= B
+        E = np.exp(-B*dt)
+
+        if self._out is not None:
+            np.mul(__x__,E,self._out)
+            __x__ = self._out
+        elif self._in_out is not None:
+            __x__ *=  E
+        else:
+            __x__ =  __x__*E
+        __x__ +=  AB
+        AB *= E
+        __x__ -= AB
         return __x__
 
-
     def select(self, method = 'Forward Euler'):
-        '''
+        """
         Select evaluation method.
 
         **Parameters**
 
-        method : 
+        method :
             * 'Forward Euler'
             * 'Runge Kutta 2'
             * 'Runge Kutta 4'
@@ -301,7 +322,7 @@ class DifferentialEquation(Definition):
 
         Exponential Euler method is only available for equation of type:
         dy/dt = A+(B)*y with A,B being valid python expression.
-        '''
+        """
 
         if method == 'Forward Euler':
             self.__method__ = self._forward_euler
@@ -320,16 +341,17 @@ class DifferentialEquation(Definition):
                 'Unknown evaluation method (%s)' % method
 
     def __call__(self, __x__, dt, *args, **kwargs):
-        '''
-        Evaluate __x__(t+dt)
-        
-        **Parameters**
+        """
+        Evaluate __x__ at time t+dt.
 
-        __x__ : float
-            Variable that need to be evaluated
-
-        dt : float
+        :param __x__:
+             Variable that need to be evaluated
+        :param dt:
             Elementaty time step
+        :param list args:
+            Optional arguments
+        :param dict kwargs:
+            Optional keyword arguments
 
         **Notes**
 
@@ -342,20 +364,16 @@ class DifferentialEquation(Definition):
         >>> eq = DifferentialEquation('dy/dt = a+b*y')
         >>> y = eq.evaluate(y, 0.01, 1, 2)     # a=1, b=2
         >>> y = eq.evaluate(y, 0.01, b=1, a=2) # a=2, b=1
-        '''
+        """
         return self.__method__ (__x__, dt, *args, **kwargs)
 
-
     def evaluate(self, __x__, dt, *args, **kwargs):
-        '''
+        """
         Evaluate __x__(t+dt)
-        
-        **Parameters**
 
-        __x__ : float
-            Variable that need to be evaluated
-
-        dt : float
+        :param __x__:
+             Variable that need to be evaluated
+        :param dt:
             Elementaty time step
 
         **Notes**
@@ -369,36 +387,38 @@ class DifferentialEquation(Definition):
         >>> eq = DifferentialEquation('dy/dt = a+b*y')
         >>> y = eq.evaluate(y, 0.01, 1, 2)     # a=1, b=2
         >>> y = eq.evaluate(y, 0.01, b=1, a=2) # a=2, b=1
-        '''
+        """
         return self.__method__ (__x__, dt, *args, **kwargs)
 
 
+
 # -----------------------------------------------------------------------------
 if __name__ == '__main__':
-    t, dt = 10.0, 0.01
-    eq = DifferentialEquation('dy/dt = a + (b)*y''')
-    #eq = DifferentialEquation('dy/dt = a + b*y''')
+    import numpy as np
+    
+    t, dt = 1.0, 0.001
+    eq = DifferentialEquation('dz/dt=a + (b)*z''')
 
     eq.select('Forward Euler')
-    y = 0.0
+    y = 1.0
     for i in range(int(t/dt)):
-        y = eq(y, dt, a=1, b=1)
+        y = eq(y, dt, a=0, b=1)
     print 'Forward Euler:      ', y
 
     eq.select('Runge Kutta 2')
-    y = 0.0
+    y = 1.0
     for i in range(int(t/dt)):
-        y = eq(y, dt, a=1, b=1)
+        y = eq(y, dt, a=0, b=1)
     print 'Runge Kutta 2:      ', y
 
     eq.select('Runge Kutta 4')
-    y = 0.0
+    y = 1.0
     for i in range(int(t/dt)):
-        y = eq(y, dt, a=1, b=1)
+        y = eq(y, dt, a=0, b=1)
     print 'Runge Kutta 4:      ', y
     
     eq.select('Exponential Euler')
-    y = 0.0
+    y = 1.0
     for i in range(int(t/dt)):
-        y = eq(y, dt, a=1, b=1)
+        y = eq(y, dt, a=0, b=1)
     print 'Exponential Euler:  ',y
