@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL license and that you accept its terms.
 # -----------------------------------------------------------------------------
-'''
+"""
 A group object represents a multidimensional, homogeneous group of contiguous
 numpy arrays.  An associated data-type object describes the format of each
 element in the group (its byte-order, how many bytes it occupies in memory,
@@ -38,7 +38,7 @@ whether it is an integer or a floating point number, etc.).
 
 A group is very similar to a numpy record array and those not familiar should
 have a look at numpy first.
-'''
+"""
 import inspect
 import numpy as np
 from model import Model
@@ -53,14 +53,14 @@ class GroupException(Exception):
 
 
 class Group(object):
-    '''
+    """
     A group object represents a multidimensional, homogeneous group of
     contiguous numpy arrays.  An associated data-type object describes the
     format of each element in the group (its byte-order, how many bytes it
     occupies in memory, whether it is an integer or a floating point number,
     etc.).
 
-    A group is very similar to a numpy record array and those not familiar 
+    A group is very similar to a numpy record array and those not familiar
     should have a look at numpy first.
 
     **See also**
@@ -71,16 +71,16 @@ class Group(object):
     * :meth:`dana.zeros_like` : Return a group of zeros with shape and type of input.
     * :meth:`dana.ones_like` : Return a group of ones with shape and type of input.
     * :meth:`dana.empty_like` : Return a empty group with shape and type of input.
-    '''
+    """
 
     def __init__(self, shape=(), dtype=float, model=None, fill=0.0, base=None):
-        '''
+        """
         Creates a new group
 
         Groups should be constructed using `ones`, `zeros` or `empty` (refer to
         the ``See also`` section above). The parameters given here describe a
         low-level method for instantiating a group.
-      
+
         **Parameters**
 
         shape : tuple of ints
@@ -94,7 +94,7 @@ class Group(object):
 
         fill : scalar
             Fill value to be used to fill group fields
-        '''
+        """
 
         # Model is prevalent over dtype
         if model is not None or (type(dtype) is str and model is None):
@@ -103,11 +103,7 @@ class Group(object):
             elif type(dtype) is str and model is None:
                 model = Model(dtype)
             dtype = []
-            for eq in model._diff_equations:
-                dtype.append((eq._varname, eq._dtype))
-            for eq in model._equations:
-                dtype.append((eq._varname, eq._dtype))
-            for eq in model._declarations:
+            for eq in model:
                 dtype.append((eq._varname, eq._dtype))
         else:
             model = Model('')
@@ -168,41 +164,26 @@ class Group(object):
 
 
     def setup(self, namespace=None):
-        '''
-        '''
+        """
+        """
         
-        # Get unknown constants (and only them) from upper frame
-        if namespace is None:
-            namespace = {}
-        numpy_ns = dir(np)
-        unknown = {}
-        for eq in self._model._diff_equations:
-            eq.parse(known_variables = self._keys)
+        namespace = namespace or {}
+        variables = []
+
+        for eq in self._model:
+            eq.setup()
             namespace[eq._varname] = self[eq._varname]
-        for eq in self._model._equations:
-            eq.parse(known_variables = self._keys)
-            namespace[eq._varname] = self[eq._varname]
-        for eq in self._model._declarations:
-            eq.parse()
-            namespace[eq._varname] = self[eq._varname]
-        for eq in self._model._diff_equations:
-            for var in eq._variables:
-                if var not in numpy_ns and var not in namespace \
-                        and var not in unknown:
-                    unknown[var] = None
-        for eq in self._model._equations:
-            for var in eq._variables:
-                if var not in numpy_ns and var not in namespace.keys() \
-                        and var not in unknown:
-                    unknown[var] = None
-        for i in range(1,len(inspect.stack())):
-            frame = inspect.stack()[i][0]
-            for name in unknown:
-                if name in frame.f_globals.keys(): # and name not in namespace.keys():
-                    unknown[name] =  frame.f_globals[name]
-                    namespace[name] =  frame.f_globals[name]
+            variables.extend(eq._variables)
+
+        for name in variables:
+            for i in range(1,len(inspect.stack())):
+                frame = inspect.stack()[i][0]
+                name = name.split('.')[0]
+                if name in frame.f_locals.keys() and name not in namespace:
+                    namespace[name] = frame.f_locals[name]
+                    break
+
         self._namespace = namespace
-        self._namespace.update(globals())
 
         # Make sure all masked units are set to 0
         if hasattr(self,'mask'):
@@ -222,12 +203,19 @@ class Group(object):
 
 
     def propagate(self):
+
+        targets = []
+        for connection in self._connections:
+            target = connection._actual_target
+            if id(target) not in targets:
+                target[...] = 0
+                targets.append(id(target))
         for connection in self._connections:
            connection.propagate()
 
 
     def evaluate(self, dt=1, update=True):
-        '''
+        """
         Evaluate group state
 
         **Parameters**
@@ -236,50 +224,20 @@ class Group(object):
             Elementary time step
         update: bool
             Whether to immediately make computed values public
-        '''
+        """
 
         self._namespace['dt'] = dt
 
-        # for connection in self._connections:
-        #    connection.propagate()
-
-        # # Differential equations
-        # for eq in self._model._diff_equations:
-        #     self._saved[eq._varname][...] = self[eq._varname] #.copy()
-        #     self._namespace[eq._varname] = self[eq._varname]
-        # for eq in self._model._diff_equations:
-        #     args = [self._saved[eq._varname],dt]+ \
-        #            [self._namespace[var] for var in eq._variables]
-        #     eq.evaluate(*args)
-        # for eq in self._model._diff_equations:
-        #     self[eq._varname][...] = self._saved[eq._varname]
-
-        # # Equations
-        # for eq in self._model._equations:
-        #     self._saved[eq._varname][...] = self[eq._varname] #.copy()
-        #     self._namespace[eq._varname] = self[eq._varname]
-        # for eq in self._model._equations:
-        #     args = [self._namespace[var] for var in eq._variables]
-        #     self._saved[eq._varname][...] = eq.evaluate(*args) #*self._mask
-        # for eq in self._model._equations:
-        #     self[eq._varname][...] = self._saved[eq._varname]
-
-
-        # # Make sure all masked units are set to 0
-        # if hasattr(self,'mask'):
-        #     for key in self._data.keys():
-        #         self._data[key] *= self.mask
-
-
         # Differential equations
         for eq in self._model._diff_equations:
-            self._saved[eq._varname][...] = self[eq._varname]
             self._namespace[eq._varname] = self[eq._varname]
 
         for eq in self._model._diff_equations:
-            args = [self._saved[eq._varname],dt]+ \
+            args = [self[eq._varname],dt]+ \
                    [self._namespace[var] for var in eq._variables]            
+            eq._out = self._saved[eq._varname]
             eq.evaluate(*args)
+            # self._saved[eq._varname] = eq.evaluate(*args)
 
         # Make newly computed values available to equations below (and only to them)
         for eq in self._model._diff_equations:
@@ -287,31 +245,33 @@ class Group(object):
 
         # Equations
         for eq in self._model._equations:
-            self._saved[eq._varname][...] = self[eq._varname]
             self._namespace[eq._varname] = self[eq._varname]
         for eq in self._model._equations:
             args = [self._namespace[var] for var in eq._variables]
-            self._saved[eq._varname][...] = eq.evaluate(*args) #*self._mask
+            self._saved[eq._varname][...] = eq.evaluate(*args)
+
             # Make results available to subsequent equations
             self._namespace[eq._varname] = self._saved[eq._varname]
 
         # Make sure all masked units are set to 0
-        if hasattr(self,'mask'):
-            for key in self._data.keys():
-                self._saved[key] *= self.mask
+#        if hasattr(self,'mask'):
+#            for key in self._data.keys():
+#                self._saved[key] *= self.mask
 
         if update:
             self.update()
 
 
     def update(self):
-        '''
+        """
         Update group state by making public previously computed values
-        '''
-        for eq in self._model._diff_equations:
-            self._data[eq._varname][...] = self._saved[eq._varname]
-        for eq in self._model._equations:
-            self._data[eq._varname][...] = self._saved[eq._varname]
+        """
+
+        self._data, self._saved = self._saved, self._data
+#        for eq in self._model._diff_equations:
+#            self._data[eq._varname][...] = self._saved[eq._varname]
+#        for eq in self._model._equations:
+#            self._data[eq._varname][...] = self._saved[eq._varname]
         
 
     def learn(self, dt=1):
@@ -321,32 +281,33 @@ class Group(object):
 
 
     def run(self, dt=1):
-         ''' '''
+         """ """
          self.propagate()
-         self.evaluate(dt)
+         self.evaluate(dt, update=False)
+         self.update()
          self.learn(dt)
 
 
     def item(self):
-        '''
+        """
         Copy the first element of group to a standard Python scalar and return
         it. The group must be of size one.
-        '''
+        """
         return self._data[self._data.keys()[0]]
 
  
 
     def ravel(self):
-        '''
+        """
         Return a flattened group.
-    
+
         A 1-D group, containing the elements of the group, is returned.
-        '''
+        """
         return self.reshape( (self.size,) )
 
 
     def reshape(self, shape):
-        '''
+        """
         Gives a new shape to the group without changing its data.
 
         **Parameters**
@@ -368,7 +329,7 @@ class Group(object):
         >>> g = group([[1,2,3], [4,5,6]])
         >>> g.reshape(6)
         group([1, 2, 3, 4, 5, 6])
-        '''
+        """
         G = Group(shape=(), dtype=self.dtype)
         for key in G.keys:
             G.data[key] = self.data[key].reshape(shape)
@@ -377,14 +338,14 @@ class Group(object):
 
 
     def __len__(self):
-        ''' x.__len__() <==> len(x) '''
+        """ x.__len__() <==> len(x) """
         if self.shape:
             return self.shape[0]
         raise TypeError, 'len() of unsized object'
 
 
     def __getattr__(self, key):
-        ''' '''
+        """ """
         if key in self._keys:
             return self._data[key]
         else:
@@ -392,7 +353,7 @@ class Group(object):
 
 
     def __setattr__(self, key, value):
-        ''' '''
+        """ """
         if key in self._keys:
             self._data[key][...] = value
         else:
@@ -400,14 +361,13 @@ class Group(object):
 
 
     def __call__(self, keys):
-        ''' '''
+        """ """
         return self.subgroup(keys)
 
 
     def subgroup(self, key):
-        ''' '''
-        dtype = []
-        dtype.append( (key, self._dtype[key]) )
+        """ """
+        dtype = [(key, self._dtype[key])]
         G = Group(shape=self.shape, dtype=dtype, base=self.base or self)
         G.data[key] = self.data[key]
         #G._base = self.base or self
@@ -442,7 +402,7 @@ class Group(object):
 
 
     def __getitem__(self, key):
-        ''' '''
+        """ """
 
         if type(key) is str:
             if key in self._keys:
@@ -497,8 +457,8 @@ class Group(object):
                 self._keys = np.dtype(dtype).names
                 return
             elif type(value) is np.ndarray:
-                if value.size == self.size and \
-                        value.dtype.names == None:
+                if value.size == self.size and\
+                   value.dtype.names is None:
                     self._data[key] = value.reshape(self.shape)
                     dtype = [(name, self.dtype[i])
                              for i, name in enumerate(self.dtype.names)]
@@ -569,15 +529,15 @@ class Group(object):
 
 
     def asarray(self):
-        ''' Return a ndarray copy of this group '''
+        """ Return a ndarray copy of this group """
 
         return np.array(self, dtype=self.dtype)
 
 
     def as_masked_array(self):
-        ''' Return a masked ndarray copy of this group '''
+        """ Return a masked ndarray copy of this group """
 
-        if hasattr(self,'mask') and self.mask != None:
+        if hasattr(self,'mask') and self.mask is not None:
             mask = self.mask
         else:
             mask = 1
@@ -592,26 +552,26 @@ class Group(object):
 
 
     def __str__(self):
-        ''' x.__str__() <==> str(x) '''
-        if hasattr(self,'mask') and self.mask != None:
+        """ x.__str__() <==> str(x) """
+        if hasattr(self,'mask') and self.mask is not None:
             return str(self.as_masked_array())
         else:
             return np.array_str(self)
 
 
     def __repr__(self):
-        ''' x.__repr__() <==> repr(x) '''
-        if hasattr(self,'mask') and self.mask != None:
+        """ x.__repr__() <==> repr(x) """
+        if hasattr(self,'mask') and self.mask is not None:
             return repr(self.as_masked_array())
         else:
             return np.array_repr(self)
 
 
     def _get_shape(self):
-        '''Get group shape'''
+        """Get group shape"""
         return self._shape
     def _set_shape(self, shape):
-        '''Set group shape'''
+        """Set group shape"""
         for key in self._dtype.names:            
             self._data[key].shape = shape
         self._shape = shape
@@ -620,49 +580,49 @@ class Group(object):
 
 
     def _get_size(self):
-        '''Get group size'''
+        """Get group size"""
         return self._data.values()[0].size
     size = property(_get_size,
                     doc = '''Number of elements in the group.''')
 
 
     def _get_base(self):
-        '''Get group base'''
+        """Get group base"""
         return self._base
     base = property(_get_base,
                     doc = '''Base group.''')
 
 
     def _get_dtype(self):
-        '''Get group dtype'''
+        """Get group dtype"""
         return self._dtype
     dtype = property(_get_dtype,
                      doc='''Data-type for the group.''')
 
 
     def _get_data(self):
-        '''Get group data'''
+        """Get group data"""
         return self._data
     data = property(_get_data,
                     doc='''Group data (list of arrays)''')
 
 
     def _get_keys(self):
-        '''Get group keys'''
+        """Get group keys"""
         return self._keys
     keys = property(_get_keys,
                     doc='''Group keys''')
 
 
     def _get_connections(self):
-        '''Get group connections'''
+        """Get group connections"""
         return self._connections
     connections = property(_get_connections,
                            doc='''Group connections''')
 
 
     def _get_model(self):
-        '''Get group model'''
+        """Get group model"""
         return self._model
     model = property(_get_model,
                     doc='''Group model''')
